package roboter;

import java.util.Random;

/** Objekte der Klasse Flieband sind Bestandteile von Objekten der Klasse Roboter.
 * Sie transportieren Objekte der Klasse Kugel, die die Greifhand des Roboters aufnehmen kann.
 */
public class Fliessband{
    /** maximale Anzahl der Kugeln auf dem Flieband
     */
    private int laenge;
    /** Feld, das auf die Kugelobjekte auf dem Flieband verweist
     */
    private Kugel[] kugel;
    /** Zufallsgenerator zur Erzeugung der Farben und der Nummern neuer Kugeln
     */
    private Random random;
    /** Nummer der nchsten Kugel, die neu links auf das Flieband gesetzt wird
     */
    private int aktuelleNummer;
    /** Winkelstellung des Roboterarms in Grad, unter dem die Greifhand des Roboters 
     * auf der ersten Kugel ist. Dieses Attribut ist eine Polarkoordinate zur Beschreibung 
     * der Position des Fliebandes.
     */
    private int relativWinkelRoboter;
    /** zweite Polarkoordinate; Abstand der ersten Kugel vom Drehpunkt des Roboterarmes
     * als Anteil der Fensterbreite 
     */
    private double armLaenge;

    /** Konstruktor
     * bergabe der maximalen Kugelzahl und der Position des Fliebandes gegenber dem Drehpunkt 
     * und der Anfangswerte fr die Attribute relativWinkelRoboter und armLaenge
     */
    public Fliessband(int l, int relativWinkel, double aL){   
        random = new Random();    
        laenge = l;
        relativWinkelRoboter = relativWinkel;
        armLaenge = aL;
        kugel = new Kugel[laenge];
        aktuelleNummer = 0;
        for (int i = 0; i < laenge; i++){
            kugel[i] = neueKugel();
        }   
    }
    /** Methode, die ein neues Kugelobjekt zuflliger Farbe erzeugt
     * und dieses an den Nutzer zurckgibt
     */
    public Kugel neueKugel(){
        aktuelleNummer = aktuelleNummer + 1;
        String s = "";
        int zufallszahl = random.nextInt(5);
        switch(zufallszahl){
            case 0: s = "rot"; break;
            case 1: s = "magenta"; break;
            case 2: s = "grn"; break;
            case 3: s = "blau"; break;
            case 4: s = "wei"; break;
        }
        return new Kugel(aktuelleNummer, s);
    }
    /** Die erste Kugel auf dem Flieband wird entfernt und das entsprechende Objekt 
     * an den Nutzer zurckgegeben.
     */
    public Kugel kugelEntfernen(){
        Kugel k = kugel[0];
        kugel[0] = null;
        return k;
    }
    /** Alle Kugeln auf dem Flieband werden um eine Position nach rechts transportiert.
     * Ist auf der Position ganz rechts eine Kugel, fllt sie vom Band. Auf die Position
     * ganz links auf dem Band wird eine neu erzeugte Kugel gesetzt.
     */
    public void kugelnTransportieren(){
        for (int i = 0; i < laenge-1; i++){
            kugel[i] = kugel[i+1];
        }
        kugel[laenge -1] = neueKugel();
    }
    /** Gibt die Kugel auf der (i+1)-ten Position (von rechts gezhlt) zurck.
     */
    public Kugel kugelGeben(int i){
        return kugel[i];
    }
    /** Gibt die maximal mgliche Zahl an Kugeln auf dem Flieband zurck.
     */
    public int laengeGeben(){
        return laenge;
    }
    /** Gibt die Winkelstellung des Roboterarms in Grad, wenn sich die Greifhand auf der
     * ersten Kugel befindet, zurck.
     */
    public int relativWinkelRoboterGeben(){
        return relativWinkelRoboter;
    }
    /** Gibt die Lnge des Roboterarms als Anteil der Fensterbreite zurck.
     */
    public double armLaengeGeben(){
        return armLaenge;
    }
}